SUBROUTINE GPPREF(KPROMA,KSTART,KPROF,KFLEV,PVAH,PVBH,PALPH,PRESH,PRESF)

!**** *GPPREF* - Computes full level pressure

!     Purpose.
!     --------
!           Computes pressures at half and full model levels.

!**   Interface.
!     ----------
!        *CALL* *GPPREF(...)

!        Explicit arguments :
!        --------------------
!                              KPROMA :  dimensioning
!                              KSTART :  start of work
!                              KPROF  :  depth of work
!                              KFLEV     : vert. dimensioning
!                              PVAH(KFLEV),PVBH(KFLEV)- vertical coordinate
!                              PALPH (KPROMA,KFLEV)  - COEFF OF THE HYDROST
!                              PRESH(KPROMA,0:KFLEV) - HALF LEVEL PRESSURE
!                              PRESF(KPROMA,KFLEV)   - FULL LEVEL PRESSURE
!
!        Implicit arguments :  NONE.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.  None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!                                PHk*ln(PHk) - PHk-1*ln(PHk-1)
!     Full level P: ln(PFk) = [ ------------------------------- - 1. ]
!                                        PHk - PHk-1

!     which simplifies to:  PFk = Pk+1/2 * exp(-ALPHA)

!     In case of NDLNPR=1 it becomes even simpler (no need of LAPRXP any
!     more in principle !) :
!                           PFk = Pk+1/2 * (1.-ALPHA) except at the top
!     level :
!                           PF1 = P1.5 / (2+Cv/R)

!     Author.
!     -------
!        Erik Andersson, Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 92-11-23
!        Modified : 95-01-31 by Radmila Bubnova: correction in the case
!                            of the other approximation of d (ln p).
!        Modified : 00-11-22 by Agathe Untch: modifications for vertical
!                            finite elements
!        01-Oct-2003 M. Hamrud   CY28 Cleaning.
!        15-Nov-2004 K. Yessad   Improve the hierarchy of tests.
!        15-Feb-2005 K. Yessad   ZTOPPRES becomes TOPPRES.
!        02-Jan-2006 J. Masek    Adapted for use with flux.
!     ------------------------------------------------------------------

USE MOD_AUX, ONLY: LAPRXPK, NDLNPR, RCVD, RD, TOPPRES

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER,INTENT(IN)    :: KPROMA 
INTEGER,INTENT(IN)    :: KFLEV 
INTEGER,INTENT(IN)    :: KSTART 
INTEGER,INTENT(IN)    :: KPROF 
REAL                  :: PVAH(0:KFLEV) ! Argument NOT used
REAL                  :: PVBH(0:KFLEV) ! Argument NOT used
REAL   ,INTENT(IN)    :: PALPH(KPROMA,KFLEV) 
REAL   ,INTENT(IN)    :: PRESH(KPROMA,0:KFLEV) 
REAL   ,INTENT(OUT)   :: PRESF(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER :: IFIRST, JLEV, JLON
REAL :: ZMUL

!     ------------------------------------------------------------------

!*       1.    Level to begin normal computations
!              ----------------------------------

! This is introduced to allow the use of GPPREF without the implicit
! assumption that the top level input for pressure is 0 hPa.
! This restriction is only necessary in the case of use of NDLNPR=1.

IF (NDLNPR == 1) THEN
  IF(PRESH(KSTART,0) <= TOPPRES)THEN
    IFIRST=2
  ELSE
    IFIRST=1
    DO JLON=KSTART,KPROF
      IF(PRESH(JLON,0) <= TOPPRES)THEN
        IFIRST=2
        EXIT
      ENDIF
    ENDDO
  ENDIF
ENDIF

!     ------------------------------------------------------------------

!*       2.    COMPUTES FULL LEVEL PRESSURES.
!              ------------------------------

IF (NDLNPR == 0) THEN
  IF (LAPRXPK) THEN
    DO JLEV=1,KFLEV
      DO JLON=KSTART,KPROF
        PRESF(JLON,JLEV)=(PRESH(JLON,JLEV-1)+PRESH(JLON,JLEV))*0.5
      ENDDO
    ENDDO
  ELSE
    DO JLEV=1,KFLEV
      DO JLON=KSTART,KPROF
        PRESF(JLON,JLEV)=EXP(-PALPH(JLON,JLEV))*PRESH(JLON,JLEV)
      ENDDO
    ENDDO
  ENDIF
ELSEIF (NDLNPR == 1) THEN
  DO JLEV=IFIRST,KFLEV
    DO JLON=KSTART,KPROF
      PRESF(JLON,JLEV)=(1.0-PALPH(JLON,JLEV))*PRESH(JLON,JLEV)
    ENDDO
  ENDDO
  ZMUL=1.0/(2.0+RCVD/RD)
  DO JLEV=1,IFIRST-1
    DO JLON=KSTART,KPROF
      PRESF(JLON,JLEV)=PRESH(JLON,JLEV)*ZMUL
    ENDDO
  ENDDO
ENDIF

!     ------------------------------------------------------------------

END SUBROUTINE GPPREF
