SUBROUTINE GPXYB(KPROMA,KSTART,KPROF,KFLEV,PVDELB,PVC,&
 & PRES,PDELP,PRDELP,PLNPR,PALPH,PRTGR,&
 & PRPRES,PRPP)  

!**** *GPXYB* - Computes auxillary arrays

!     Purpose.
!     --------
!           COMPUTES AUXILLARY ARRAYS RELATED TO THE HYBRID COORDINATE

!**   Interface.
!     ----------
!        *CALL* *GPXYB(..)

!        Explicit arguments :
!        --------------------
!     KPROMA : dimensioning
!     KSTART : start of work
!     KPROF  : depth of work
!     KFLEV     : vert. dimensioning

!     PVDELB(KPROMA,0:KFLEV) : related to vert. coordinate        (input)
!     PVC   (KPROMA,0:KFLEV) :  "     "      "     "    "         (input)
!     PRES (KPROMA,0:KFLEV)  : HALF LEVEL PRESSURE                (input)
!     PDELP (KPROMA,KFLEV)   : PRESSURE DIFFERENCE ACROSS LAYERS  (output)
!     PRDELP(KPROMA,KFLEV)   : THEIR INVERSE                      (output)
!     PLNPR (KPROMA,KFLEV)   : LOGARITHM OF RATIO OF PRESSURE     (output)
!     PALPH (KPROMA,KFLEV)   : COEFFICIENTS OF THE HYDROSTATICS   (output)
!     PRTGR (KPROMA,KFLEV)   : FOR PRES. GRAD. TERM AND ENE. CONV.(output)
!                              ((rssavnabla prehyd/prehyd)_[layer]
!                              = prtgr_[layer] * (rssavnabla prehyds))
!     PRPRES(KPROMA,KFLEV)   : inverse of HALF LEVEL PRESSURE     (output)
!     PRPP  (KPROMA,KFLEV)   : inverse of PRES(J)*PRES(J-1)       (output)

!        Implicit arguments :  None.
!        --------------------

!     Method.
!     -------
!        See documentation

!     Externals.      None.
!     ----------

!     Reference.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     Author.
!     -------
!        Mats Hamrud and Philippe Courtier  *ECMWF*

!     Modifications.
!     --------------
!        Original : 88-02-04
!        Modified : 94-10-11 by Radmila Bubnova: correction in the case
!                            of the other approximation of d (ln p).
!        Modified : 99-06-04 Optimisation   D.SALMOND
!        Modified : 02-03-08 K. YESSAD: consistent discretisations of
!                    "alpha" (PALPH) and "prtgr" (PRTGR)
!                    for finite element vertical discretisation
!                    to allow model to run with MF-physics and DDH.
!        Modified : 03-07-07 J. Hague:  Replace divides with reciprocal
!        01-Oct-2003 M. Hamrud   CY28 Cleaning.
!        15-Feb-2005 K. Yessad   ZTOPPRES becomes TOPPRES.
!        02-Jan-2006 J. Masek    Adapted for use with flux.
!     ------------------------------------------------------------------

USE MOD_AUX, ONLY: NDLNPR, RCVD, RD, RHYDR0, TOPPRES

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER,INTENT(IN)    :: KPROMA 
INTEGER,INTENT(IN)    :: KFLEV 
INTEGER,INTENT(IN)    :: KSTART 
INTEGER,INTENT(IN)    :: KPROF 
REAL   ,INTENT(IN)    :: PVDELB(KFLEV) 
REAL   ,INTENT(IN)    :: PVC(KFLEV) 
REAL   ,INTENT(IN)    :: PRES(KPROMA,0:KFLEV) 
REAL   ,INTENT(INOUT) :: PDELP(KPROMA,KFLEV) 
REAL   ,INTENT(INOUT) :: PRDELP(KPROMA,KFLEV) 
REAL   ,INTENT(INOUT) :: PLNPR(KPROMA,KFLEV) 
REAL   ,INTENT(OUT)   :: PALPH(KPROMA,KFLEV) 
REAL   ,INTENT(OUT)   :: PRTGR(KPROMA,KFLEV) 
REAL   ,INTENT(OUT)   :: PRPRES(KPROMA,KFLEV) 
REAL   ,INTENT(INOUT) :: PRPP(KPROMA,KFLEV) 

!     ------------------------------------------------------------------

INTEGER :: IFIRST, JLEV, JLON, JJ, JTEMP, JM

REAL :: ZRPRES(KPROMA,2)

!     ------------------------------------------------------------------

!*       0.    Level to begin normal computations
!              ----------------------------------

! This is introduced to allow the use of GPXYB without the implicit
! assumption that the top level input for pressure is 0 hPa. This
! is used in the surface observation operators where you do not want
! to compute geopotential at all model levels.
! The first block if is for economy (no do loop start up) and the second
! for safety.
IF(PRES(KSTART,0) <= TOPPRES)THEN
  IFIRST=2
ELSE
  IFIRST=1
  DO JLON=KSTART,KPROF
    IF(PRES(JLON,0) <= TOPPRES)then
      IFIRST=2
      EXIT
    ENDIF
  ENDDO
ENDIF
!     ------------------------------------------------------------------

!*       1.    COMPUTES EVERYTHING.
!              --------------------

IF(NDLNPR == 0) THEN

  JJ=1
  JM=2
  DO JLON=KSTART,KPROF
    ZRPRES(JLON,JM)=1.0/PRES(JLON,IFIRST-1)
  ENDDO
  DO JLEV=IFIRST,KFLEV
    DO JLON=KSTART,KPROF
      ZRPRES(JLON,JJ)=1.0/PRES(JLON,JLEV)
      PDELP (JLON,JLEV)=PRES(JLON,JLEV)-PRES(JLON,JLEV-1)
      PRDELP(JLON,JLEV)=1.0/PDELP(JLON,JLEV)
      PLNPR (JLON,JLEV)=LOG(PRES(JLON,JLEV)*ZRPRES(JLON,JM))
      PRPRES(JLON,JLEV)=ZRPRES(JLON,JJ)
      PALPH (JLON,JLEV)=1.0-PRES(JLON,JLEV-1)*PRDELP(JLON,JLEV)&
       & *PLNPR(JLON,JLEV)  
      PRPP  (JLON,JLEV)=ZRPRES(JLON,JJ)*ZRPRES(JLON,JM)
      PRTGR (JLON,JLEV)=PRDELP(JLON,JLEV)&
       & *(PVDELB(JLEV)+PVC(JLEV)*PLNPR(JLON,JLEV)*PRDELP(JLON,&
       & JLEV))  
    ENDDO
    JTEMP=JM
    JM=JJ
    JJ=JTEMP
  ENDDO
  DO JLEV=1,IFIRST-1
    DO JLON=KSTART,KPROF
      PDELP (JLON,JLEV)=PRES(JLON,JLEV)-PRES(JLON,JLEV-1)
      PRDELP(JLON,JLEV)=1.0/PDELP(JLON,JLEV)
      PLNPR (JLON,JLEV)=LOG(PRES(JLON,1)/TOPPRES)
      PRPRES(JLON,JLEV)=1.0/PRES(JLON,1)
      PALPH (JLON,JLEV)=RHYDR0
      PRPP  (JLON,JLEV)=1.0/(PRES(JLON,1)*TOPPRES)
      PRTGR (JLON,JLEV)=PRDELP(JLON,JLEV)*PVDELB(JLEV)
    ENDDO
  ENDDO

ELSEIF(NDLNPR == 1) THEN

  DO JLEV=IFIRST,KFLEV
    DO JLON=KSTART,KPROF
      PDELP (JLON,JLEV)=PRES(JLON,JLEV)-PRES(JLON,JLEV-1)
      PRDELP(JLON,JLEV)=1.0/PDELP(JLON,JLEV)
      PRPP  (JLON,JLEV)=1.0/(PRES(JLON,JLEV)*PRES(JLON,JLEV-1))
      PLNPR (JLON,JLEV)=PDELP(JLON,JLEV)*SQRT(PRPP(JLON,JLEV))
      PALPH (JLON,JLEV)=1.0-PRES(JLON,JLEV-1)*PRDELP(JLON,JLEV)&
       & *PLNPR(JLON,JLEV)  
      PRTGR (JLON,JLEV)=PRDELP(JLON,JLEV)&
       & *(PVDELB(JLEV)+PVC(JLEV)*PLNPR(JLON,JLEV)*PRDELP(JLON,&
       & JLEV))  
      PRPRES(JLON,JLEV)=1.0/PRES(JLON,JLEV)
    ENDDO
  ENDDO

  DO JLEV=1,IFIRST-1
    DO JLON=KSTART,KPROF
      PDELP (JLON,JLEV)=PRES(JLON,JLEV)
      PRDELP(JLON,JLEV)=1.0/PDELP(JLON,JLEV)
      PLNPR (JLON,JLEV)=2.0+RCVD/RD
      PALPH (JLON,JLEV)=1.0
      PRTGR (JLON,JLEV)=PRDELP(JLON,JLEV)*PVDELB(JLEV)
      PRPRES(JLON,JLEV)=1.0/PRES(JLON,1)
      PRPP  (JLON,JLEV)=(PLNPR(JLON,JLEV)*PRDELP(JLON,JLEV))**2
    ENDDO
  ENDDO

ENDIF

! (PLNPR(JLON,1) AND PRPP(JLON,1) ARE A PRIORI NOT USED LATER)

!     ------------------------------------------------------------------

END SUBROUTINE GPXYB
