program flux

! -----
! PURPOSE:
!   Diagnostic tool for 2D vertical plane ALADIN, performs FA to ASCII 
!   conversion. Works on both hydrostatic and non-hydrostatic files.
!
! DESCRIPTION:
!   This program reads historical file produced by 2D ALADIN.
!   It transforms fields from spectral space to gridpoint space.
!   Additional fields are diagnosed. Results are written into
!   ascii files using GMT format.
!
! INPUT FILES:
!   <ICMSH>  - historical file from 2D vertical plane model
!              (name specified via command line)
!   namflux  - namelist file
!
! OUTPUT FILES:
!   data_orog.gmt   - orography                                       [m]
!   data_orogm.gmt  - meridional derivative of orography              [1]
!   data_pi_S.gmt   - surface hydrostatic pressure                    [Pa]
!
!   data_p.gmt      - true pressure                                   [Pa]
!   data_pd.gmt     - non-hydrostatic pressure departure (p - pi)/pi  [1]
!   data_q.gmt      - specific humidity                               [1]
!   data_T.gmt      - temperature                                     [K]
!   data_theta.gmt  - potential temperature                           [K]
!   data_u.gmt      - U-wind                                          [m/s]
!   data_v.gmt      - V-wind                                          [m/s]
!   data_vd_HY.gmt  - hydrostatic vertical divergence dw/dz           [1/s]
!   data_vd_NH.gmt  - non-hydrostatic vertical divergence dw/dz       [1/s]
!   data_w_HY.gmt   - hydrostatic vertical velocity                   [m/s]
!   data_w_NH.gmt   - non-hydrostatic vertical velocity               [m/s]
!   data_z_f.gmt    - height of full levels (only for czunit = '')    [m]
!   data_z_h.gmt    - height of half levels (only for czunit = '')    [m]
!
! EXTERNALS:
!   gpgeo    (arp)
!   gppref   (arp)
!   gppreh   (arp)
!   gprcp    (arp)
!   gptet    (arp)
!   gpverdia (arp)
!   gpxyb    (arp)
!
!   facies   (xrd)
!   facile   (xrd)
!   fairme   (xrd)
!   faitou   (xrd)
!   set99    (xrd)
!
! ORIGINAL:
!   16-08-2002, J. Masek
!
! MODIFICATIONS:
!
!
! -----

use yomcst, only: rg

implicit none

! -----
! parameters
! -----

! fortran logical units for I/O
integer, parameter :: nulout =  6  ! output messages
integer, parameter :: nulnam = 10  ! namelist file
integer, parameter :: nulgmt = 11  ! GMT file
integer, parameter :: nulfa  = 12  ! FA file

! WARNING:
!   Insufficient value for any of the following parameters can 
!   cause overwriting of array bounds when calling FA routines.
!   This usually results in a strange behaviour.
!
!   Given values are default for VPP. They should be safe also
!   for workstations. See include fa/facomp.h under xrd for more
!   details.
!
! maximal dimensions of some arrays used by FA routines
integer, parameter :: jpxgeo = 256
integer, parameter :: jpxind = 1026
integer, parameter :: jpxniv = 999   ! maximum number of layers
integer, parameter :: jpxpah = 256

! -----
! functions
! -----

integer :: iargc, lnblnk

! -----
! local variables
! -----

integer :: inbari, inbarp, inimes, iret
integer :: jlev, jrof
integer :: nx, nx1, nx2, nz, nz1, nz2
integer :: ndgl, ndlon, nflevg, nmsmax, nsmax, nsefre

real :: edely, ely
real :: zcodil, zclopo, zslapo, zslopo, zprefer

character(len =  2) :: cxunit, czunit
character(len = 16) :: cnmca
character(len = 20) :: csuff
character(len = 80) :: cfile_fa, cfile_nam, cpref
character(len = 80) :: cfmt1, cfmt2e, cfmt2f, cfmtx, cfmtz

logical :: lnhdyn, lzoom
logical :: lerfa, limst, lnomf, llg

! -----
! local arrays
! -----

integer :: ifax(20)
integer :: inlopa(jpxpah)
integer :: inozpa(jpxind)

real :: vah   (0:jpxniv)
real :: vbh   (0:jpxniv)
real :: vc    (  jpxniv)
real :: vdelb (  jpxniv)
real :: zsinla(  jpxgeo)

! arrays for spectral computations
real, allocatable :: trigs (:)
real, allocatable :: zspec (:)
real, allocatable :: zspecm(:)

! 1D gridpoint fields
real, allocatable :: zorog (:)
real, allocatable :: zorogm(:)
real, allocatable :: zpim_s(:)
real, allocatable :: zx    (:)

! 2D gridpoint fields
real, allocatable :: zdelp  (:, :)
real, allocatable :: zrdelp (:, :)
real, allocatable :: zlnpr  (:, :)
real, allocatable :: zalph  (:, :)
real, allocatable :: zrtgr  (:, :)
real, allocatable :: zrpref (:, :)
real, allocatable :: zrpp   (:, :)
real, allocatable :: zcp    (:, :)
real, allocatable :: zr     (:, :)
real, allocatable :: zkap   (:, :)
real, allocatable :: zrt    (:, :)
real, allocatable :: ztheta (:, :)
real, allocatable :: zvvel  (:, :)
real, allocatable :: zphi_h (:, :)
real, allocatable :: zphil_h(:, :)
real, allocatable :: zphim_h(:, :)
real, allocatable :: zphi_f (:, :)
real, allocatable :: zpi_h  (:, :)
real, allocatable :: zpi_f  (:, :)
real, allocatable :: zp     (:, :)
real, allocatable :: zq     (:, :)
real, allocatable :: zqi    (:, :)
real, allocatable :: zql    (:, :)
real, allocatable :: zqr    (:, :)
real, allocatable :: zqs    (:, :)
real, allocatable :: zqg    (:, :)
real, allocatable :: zt     (:, :)
real, allocatable :: zu     (:, :)
real, allocatable :: zv     (:, :)
real, allocatable :: zvm    (:, :)
real, allocatable :: zpd    (:, :)
real, allocatable :: zvd    (:, :)
real, allocatable :: zvd_hy (:, :)
real, allocatable :: zw     (:, :)
real, allocatable :: zw_hy  (:, :)
real, allocatable :: zz_h   (:, :)
real, allocatable :: zz_f   (:, :)

! -----
! namelists
! -----

namelist /NAMFLUX/ lzoom, nx1, nx2, nz1, nz2,  &
  &  cpref, csuff, cxunit, czunit

! =====

! -----
! read input arguments
! -----

! check number of input arguments
if ( iargc() /= 1 ) then

  write(nulout, *)
  write(nulout, '(a)') 'USAGE:'
  write(nulout, '(a)') '  flux historical_file'
  write(nulout, *)
  call exit(1)

endif

! set arguments into variables
call getarg(1, cfile_fa)

! -----
! read namelist
! -----

! set default values
lzoom  = .FALSE.  ! switch for zoom
nx1    =    1     ! first gridpoint of zoomed area
nx2    = 1000     ! last gridpoint of zoomed area
nz1    =    1     ! first layer of zoomed area
nz2    = 1000     ! last layer of zoomed area
cpref  = 'data_'  ! prefix of output files (can contain path)
csuff  = '.gmt'   ! suffix of output files
cxunit = ''       ! units for x-coordinate
czunit = 'm'      ! units for z-coordinate

! name of namelist file
cfile_nam = 'namflux'

! open namelist
open(nulnam, file = cfile_nam, status = 'OLD', iostat = iret)
if ( iret /= 0 ) then

  write(nulout, '(a)') 'flux: Error:'
  write(nulout, '(3a)')  &
    &  '  Cannot open namelist ', cfile_nam(1:lnblnk(cfile_nam)), '!'
  call exit(1)

endif

! read namelist
read(nulnam, NAMFLUX, iostat = iret)
if ( iret /= 0 ) then

  write(nulout, '(a)') 'flux: Error:'
  write(nulout, '(3a)')  &
    &  '  Wrong namelist ', cfile_nam(1:lnblnk(cfile_nam)), '!'
  call exit(1)

endif

! close namelist
close(nulnam)

! write info message
write(nulout, *)
write(nulout, '(3a)')  &
  &  'Namelist ', cfile_nam(1:lnblnk(cfile_nam)), ' was read successfully.'
write(nulout, *)

! -----
! check units
! -----

! units for x-coordinate
if ( cxunit == '' ) then

  write(nulout, '(a)') 'Coordinate x will be gridpoint index.'
  cfmtx = 'f5.1'

elseif ( cxunit == 'm' ) then

  write(nulout, '(a)') 'Coordinate x will be in meters.'
  cfmtx = 'f9.1'

elseif ( cxunit == 'km' ) then

  write(nulout, '(a)') 'Coordinate x will be in kilometers.'
  cfmtx = 'f9.4'

else

  write(nulout, '(3a)') 'Value cxunit = ', cxunit, ' not supported!'
  call exit(1)  

endif

! units for z-coordinate
if ( czunit == '' ) then

  write(nulout, '(a)') 'Coordinate z will be level number.'
  cfmtz = 'f5.1'

elseif ( czunit == 'm' ) then

  write(nulout, '(a)') 'Coordinate z will be in meters.'
  cfmtz = 'f8.1'

elseif ( czunit == 'km' ) then

  write(nulout, '(a)') 'Coordinate z will be in kilometers.'
  cfmtz = 'f8.4'

else

  write(nulout, '(3a)') 'Value czunit = ', czunit, ' not supported!'
  call exit(1)  

endif
write(nulout, *)

! create format strings
write(cfmt1 , '(3a)') '(', cfmtx(1:lnblnk(cfmtx)), ', 1x, f10.3)'
write(cfmt2e, '(5a)') '(', cfmtx(1:lnblnk(cfmtx)), ', 1x,  &
  &  ', cfmtz(1:lnblnk(cfmtz)), ', 1x, es12.5)'
write(cfmt2f, '(5a)') '(', cfmtx(1:lnblnk(cfmtx)), ', 1x,  &
  &  ', cfmtz(1:lnblnk(cfmtz)), ', 1x, f12.4)'

! -----
! open FA file, read header
! -----

! set attributes
inimes = 0                 ! level of diagnostic messages (0, 1 or 2)
inbarp = 1
lnomf  = .TRUE.            ! FA file given via filename
lerfa  = .FALSE.           ! do not call abort if error occurs in FA routine
limst  = .FALSE.           ! do not produce statistics when closing file
cnmca  = 'CADRE.STANDARD'  ! type of header

! open file
call faitou(iret, nulfa, lnomf, cfile_fa, 'OLD', lerfa, limst,  &
  &  inimes, inbarp, inbari, cnmca)

! check return code
if ( iret /= 0 ) then

  write(nulout, '(a)') 'flux: Error:'
  write(nulout, '(3a)')  &
    &  '  Cannot open file ', cfile_fa(1:lnblnk(cfile_fa)), '!'
  call exit(1)

endif

! read FA header
call facies(cnmca, nmsmax, zslapo, zclopo, zslopo, zcodil, nsmax,  &
  &  ndgl, ndlon, inlopa, inozpa, zsinla, nflevg, zprefer, vah, vbh, llg)

! check consistency
if ( nmsmax /= 0 ) then

  write(nulout, '(a)') 'flux: Error:'
  write(nulout, '(a, i4, a)')  &
    &  '  Input file is not 2D vertical plane! (nmsmax = ', nmsmax, ')'
  call exit(1)

endif

! write size of domain
write(nulout, '(2(a, i3))')  &
  &  'Size of domain (ndgl x nflevg): ', ndgl, ' x ', nflevg

! -----
! set up zoom area
! -----

if ( lzoom ) then

  ! cut off zoom area if it exceeds original domain
  nx1 = min(ndgl  , max(1, nx1))
  nx2 = min(ndgl  , max(1, nx2))
  nz1 = min(nflevg, max(1, nz1))
  nz2 = min(nflevg, max(1, nz2))

  ! sort indices
  if ( nx1 > nx2 ) then
    nx1 = nx1 + nx2
    nx2 = nx1 - nx2
    nx1 = nx1 - nx2
  endif
  if ( nz1 > nz2 ) then
    nz1 = nz1 + nz2
    nz2 = nz1 - nz2
    nz1 = nz1 - nz2
  endif

else

  ! take whole domain
  nx1 = 1
  nx2 = ndgl
  nz1 = 1
  nz2 = nflevg

endif

! size of zoomed area
nx = nx2 - nx1 + 1
nz = nz2 - nz1 + 1

! write zoomed area
if ( lzoom ) then

  write(nulout, '(2(a, i3))')  &
    &  'Size of zoomed area:            ', nx, ' x ', nz
  write(nulout, '(2(a, i3))') '  nx1 = ', nx1, '    nz1 = ', nz1
  write(nulout, '(2(a, i3))') '  nx2 = ', nx2, '    nz2 = ', nz2

endif
write(nulout, *)

! -----
! set up geometry
! -----

! domain size and increment [m]
ely   = zsinla(14)
edely = zsinla(16)

! vertical parameters
vah(0:nflevg) = vah(0:nflevg)*zprefer
do jlev = 1, nflevg
  vc   (jlev) = vah(jlev)*vbh(jlev - 1) - vah(jlev - 1)*vbh(jlev)
  vdelb(jlev) = vbh(jlev) - vbh(jlev - 1)
enddo

! -----
! set up for spectral computations
! -----

! dimension for spectral arrays
nsefre = 4*(nsmax + 1)

! allocate auxiliary arrays
allocate(trigs (ndgl))
allocate(zspec (nsefre))
allocate(zspecm(nsefre))

! compute factors of ndgl and trigonometric functions for FFT
call set99(trigs, ifax, ndgl)

! -----
! determine lnhdyn
! -----

! try to read NH pressure departure from level 1
call facile(iret, nulfa, 'S', 1, 'PRESS.DEPART', zspec, .TRUE.)

! check return code
if ( iret == 0 ) then

  write(nulout, '(a)')  &
    &  'Reading non-hydrostatic file, setting lnhdyn = .TRUE.'

  lnhdyn = .TRUE.

else

  write(nulout, '(a)')  &
    &  'Reading hydrostatic file, setting lnhdyn = .FALSE.'

  lnhdyn = .FALSE.

endif
write(nulout, *)

! -----
! set up necessary module variables
! -----

call setup_modules(nulout, ndgl, nflevg, nsefre, nsmax,  &
  &  ely, edely, lnhdyn)

! -----
! allocate gridpoint fields
! -----

allocate(zorog (ndgl))
allocate(zorogm(ndgl))
allocate(zpim_s(ndgl))
allocate(zx    (ndgl))

allocate(zphi_f (ndgl,   nflevg))
allocate(zphi_h (ndgl, 0:nflevg))
allocate(zphil_h(ndgl, 0:nflevg))
allocate(zphim_h(ndgl, 0:nflevg))
allocate(zpi_f  (ndgl,   nflevg))
allocate(zpi_h  (ndgl, 0:nflevg))
allocate(zp     (ndgl,   nflevg))
allocate(zq     (ndgl,   nflevg))
allocate(zt     (ndgl,   nflevg))
allocate(zu     (ndgl,   nflevg))
allocate(zv     (ndgl,   nflevg))
allocate(zvm    (ndgl,   nflevg))
allocate(zz_f   (ndgl,   nflevg))
allocate(zz_h   (ndgl, 0:nflevg))

if ( lnhdyn ) then
  allocate(zpd(ndgl, nflevg))
  allocate(zvd(ndgl, nflevg))
endif

! -----
! read 1D spectral fields
! -----

! surface pressure pi_S
call facile(iret, nulfa, 'SURF', 0, 'PRESSION', zspec, .TRUE.)
call spec2real(trigs, ifax, zspec, zpi_h(:, nflevg))

! convert surface pressure if needed
if ( maxval(zpi_h(:, nflevg)) < 100. ) then

  write(nulout, '(a)') 'Converting ln(pi_S) ---> pi_S.'
  write(nulout, *)

  zpi_h(:, nflevg) = exp(zpi_h(:, nflevg))

endif

! compute derivative of surface pressure pi_S
call real2spec(trigs, ifax, zpi_h(:, nflevg), zspec)
call specder(zspec, zspecm)
call spec2real(trigs, ifax, zspecm, zpim_s)

! -----

! surface geopotential / orography
call facile(iret, nulfa, 'SPECSURF', 0, 'GEOPOTEN', zspec, .TRUE.)
call spec2real(trigs, ifax, zspec, zphi_h(:, nflevg))

! convert surface geopotential to altitude
zorog(:) = zphi_h(:, nflevg)/rg

! compute derivative of orography
call real2spec(trigs, ifax, zorog, zspec)
call specder(zspec, zspecm)
call spec2real(trigs, ifax, zspecm, zorogm)

! -----
! read 2D spectral fields
! -----

! loop through vertical levels
do jlev = 1, nflevg

  ! specific humidity
  call facile(iret, nulfa, 'S', jlev, 'HUMI.SPECIFIQUE', zspec, .TRUE.)
  call spec2real(trigs, ifax, zspec, zq(:, jlev))

  ! temperature
  call facile(iret, nulfa, 'S', jlev, 'TEMPERATURE', zspec, .TRUE.)
  call spec2real(trigs, ifax, zspec, zt(:, jlev))

  ! u wind
  call facile(iret, nulfa, 'S', jlev, 'WIND.U.PHYS', zspec, .TRUE.)
  call spec2real(trigs, ifax, zspec, zu(:, jlev))

  ! v wind
  call facile(iret, nulfa, 'S', jlev, 'WIND.V.PHYS', zspec, .TRUE.)
  call spec2real(trigs, ifax, zspec, zv(:, jlev))

  ! compute horizontal divergence
  call specder(zspec, zspecm)
  call spec2real(trigs, ifax, zspecm, zvm(:, jlev))

  ! NH fields
  if ( lnhdyn ) then

    ! non-hydrostatic pressure departure p - pi
    call facile(iret, nulfa, 'S', jlev, 'PRESS.DEPART', zspec, .TRUE.)
    call spec2real(trigs, ifax, zspec, zpd(:, jlev))

    ! unscaled vertical divergence -g.dw
    call facile(iret, nulfa, 'S', jlev, 'VERTIC.DIVER', zspec, .TRUE.)
    call spec2real(trigs, ifax, zspec, zvd(:, jlev))

  endif

enddo

! -----
! close FA file
! -----

call fairme(iret, nulfa, 'KEEP')

! -----
! diagnose additional fields
! -----

! allocate auxiliary 2D arrays
allocate(zdelp (ndgl,   nflevg))
allocate(zrdelp(ndgl,   nflevg))
allocate(zlnpr (ndgl,   nflevg))
allocate(zalph (ndgl,   nflevg))
allocate(zrtgr (ndgl,   nflevg))
allocate(zrpref(ndgl,   nflevg))
allocate(zrpp  (ndgl,   nflevg))
allocate(zqi   (ndgl,   nflevg))
allocate(zql   (ndgl,   nflevg))
allocate(zqr   (ndgl,   nflevg))
allocate(zqs   (ndgl,   nflevg))
allocate(zqg   (ndgl,   nflevg))
allocate(zcp   (ndgl,   nflevg))
allocate(zr    (ndgl,   nflevg))
allocate(zkap  (ndgl,   nflevg))
allocate(zrt   (ndgl,   nflevg))
allocate(ztheta(ndgl,   nflevg))
allocate(zvvel (ndgl,   nflevg))
allocate(zvd_hy(ndgl,   nflevg))
allocate(zw_hy (ndgl, 0:nflevg))
if ( lnhdyn ) then
  allocate(zw(ndgl, 0:nflevg))
endif

! half level pressure pi
call gppreh(ndgl, 1, ndgl, nflevg, vah, vbh, zpi_h)

! auxiliary arrays
call gpxyb(ndgl, 1, ndgl, nflevg, vdelb, vc, zpi_h, zdelp, zrdelp,  &
  &  zlnpr, zalph, zrtgr, zrpref, zrpp)

! full level pressure pi
call gppref(ndgl, 1, ndgl, nflevg, vah, vbh, zalph, zpi_h, zpi_f)

! true pressure p
if ( lnhdyn ) then
  zp(:, :) = zpd(:, :) + zpi_f(:, :)
else
  zp(:, :) = zpi_f(:, :)
endif

! R and cp
zqi(:, :) = 0.0
zql(:, :) = 0.0
zqr(:, :) = 0.0
zqs(:, :) = 0.0
zqg(:, :) = 0.0
call gprcp(ndgl, 1, ndgl, nflevg, zq, zqi, zql, zqr, zqs, zqg,  &
  &  zcp, zr, zkap)

! RT
zrt(:, :) = zr(:, :)*zt(:, :)

! potential temperature
call gptet(ndgl, 1, ndgl, nflevg, zp, zt, zkap, ztheta)

if ( lnhdyn ) then

  ! reduced NH pressure departure (p - pi)/pi
  zpd(:, :) = zpd(:, :)/zpi_f(:, :)

  ! reduced gas constant R*pi/p
  zr(:, :) = zr(:, :)*zpi_f(: ,:)/zp(:, :)

endif

! geopotential (half and full level)
call gpgeo(ndgl, 1, ndgl, nflevg, zphi_h, zphi_f, zt, zr, zlnpr, zalph)

! -----

! hydrostatic true vertical divergence

! grad(phi) on half levels
zphil_h(:, :) = 0.0
do jlev = 0, nflevg
  call real2spec(trigs, ifax, zphi_h(:, jlev), zspec)
  call specder(zspec, zspecm)
  call spec2real(trigs, ifax, zspecm, zphim_h(:, jlev))
enddo

! omega/pi (array zvvel)
call vvel(vbh, vdelb, zpim_s, zv, zvm,  &
  &  zdelp, zrdelp, zlnpr, zalph, zrtgr, zrpref, zvvel)

! vertical divergence
call gpverdia(ndgl, 1, ndgl, zu, zv, zvm, zvvel, zphil_h, zphim_h,  &
  &  zlnpr, zalph, zrt, zkap, zvd_hy)

! -----

! hydrostatic vertical velocity at surface (surface horizontal wind 
! assumed the same as on lowest full level)
zw_hy(:, nflevg) = zv(:, nflevg)*zorogm(:)

! hydrostatic vertical velocity
do jlev = nflevg, 1, -1
  zw_hy(:, jlev - 1) = zw_hy(:, jlev) +  &
    &  zvd_hy(:, jlev)*(zphi_h(:, jlev - 1) - zphi_h(:, jlev))/rg
enddo

if ( lnhdyn ) then

  ! non-hydrostatic vertical velocity
  zw(:, nflevg) = zw_hy(:, nflevg)
  do jlev = nflevg, 1, -1
    zw(:, jlev - 1) = zw(:, jlev) + zvd(:, jlev)/rg
  enddo

  ! true vertical divergence
  do jlev = 1, nflevg
    zvd(:, jlev) = zvd(:, jlev)/(zphi_h(:, jlev - 1) - zphi_h(:, jlev))
  enddo

endif

! deallocate auxiliary arrays
deallocate(trigs, zspec, zspecm)
deallocate(zpim_s)
deallocate(zdelp, zrdelp, zlnpr, zalph, zrtgr, zrpref, zrpp, zvvel)
deallocate(zqi, zql, zqr, zqs, zqg)
deallocate(zcp, zr, zkap, zphil_h, zphim_h, zrt, zvm)

! -----

! x-coordinate
if ( cxunit == '' ) then

  do jrof = 1, ndgl
    zx(jrof) = jrof
  enddo

elseif ( cxunit == 'm' ) then

  do jrof = 1, ndgl
    zx(jrof) = (jrof - 1)*edely
  enddo

elseif ( cxunit == 'km' ) then

  do jrof = 1, ndgl
    zx(jrof) = (jrof - 1)*edely/1000.
  enddo

endif

! -----

! z-coordinate
if ( czunit == '' ) then

  do jrof = 1, ndgl
    zz_h(jrof, 0) = 0.
    do jlev = 1, nflevg
      zz_h(jrof, jlev) = real(jlev)
      zz_f(jrof, jlev) = real(jlev)
    enddo
  enddo

elseif ( czunit == 'm' ) then

  zz_h = zphi_h/rg
  zz_f = zphi_f/rg

elseif ( czunit == 'km' ) then

  zz_h = zphi_h/(1000.*rg)
  zz_f = zphi_f/(1000.*rg)

endif

! -----
! write results into ascii files
! -----

! 1D fields

! surface pressure pi_S
call write_1d(nulgmt, cpref, 'pi_S', csuff, cfmt1,  &
  &  zpi_h(nx1:nx2, nflevg), zx(nx1:nx2), nx)

! orography
call write_1d(nulgmt, cpref, 'orog', csuff, cfmt1,  &
  &  zorog(nx1:nx2), zx(nx1:nx2), nx)

! derivative of orography
call write_1d(nulgmt, cpref, 'orogm', csuff, cfmt1,  &
  &  zorogm(nx1:nx2), zx(nx1:nx2), nx)

! 2D fields

! true pressure p
call write_2d(nulgmt, cpref, 'p', csuff, cfmt2f,  &
  &  zp(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
  &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

! specific humidity
call write_2d(nulgmt, cpref, 'q', csuff, cfmt2e,  &
  &  zq(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
  &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

! temperature
call write_2d(nulgmt, cpref, 'T', csuff, cfmt2f,  &
  &  zt(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
  &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

! potential temperature
call write_2d(nulgmt, cpref, 'theta', csuff, cfmt2f,  &
  &  ztheta(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
  &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

! u wind
call write_2d(nulgmt, cpref, 'u', csuff, cfmt2f,  &
  &  zu(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
  &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

! v wind
call write_2d(nulgmt, cpref, 'v', csuff, cfmt2f,  &
  &  zv(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
  &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

! hydrostatic true vertical divergence dw/dz
call write_2d(nulgmt, cpref, 'vd_HY', csuff, cfmt2e,  &
  &  zvd_hy(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
  &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

! hydrostatic vertical velocity w
call write_2d(nulgmt, cpref, 'w_HY', csuff, cfmt2e,  &
  &  zw_hy(nx1:nx2, nz1 - 1:nz2), zx(nx1:nx2),  &
  &  zz_h (nx1:nx2, nz1 - 1:nz2), nx, nz + 1)

if ( czunit == '' ) then

  ! altitude of full levels
  call write_2d(nulgmt, cpref, 'z_f', csuff, cfmt2f,  &
    &  zphi_f(nx1:nx2, nz1:nz2)/rg, zx(nx1:nx2),  &
    &  zz_f  (nx1:nx2, nz1:nz2), nx, nz)

  ! altitude of half levels
  call write_2d(nulgmt, cpref, 'z_h', csuff, cfmt2f,  &
    &  zphi_h(nx1:nx2, nz1 - 1:nz2)/rg, zx(nx1:nx2),  &
    &  zz_h  (nx1:nx2, nz1 - 1:nz2), nx, nz + 1)

endif

if ( lnhdyn ) then

  ! non-hydrostatic pressure departure (p - pi)/pi
  call write_2d(nulgmt, cpref, 'pd', csuff, cfmt2e,  &
    &  zpd(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
    &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

  ! non-hydrostatic true vertical divergence dw/dz
  call write_2d(nulgmt, cpref, 'vd_NH', csuff, cfmt2e,  &
    &  zvd(nx1:nx2, nz1:nz2), zx(nx1:nx2),  &
    &  zz_f(nx1:nx2, nz1:nz2), nx, nz)

  ! non-hydrostatic vertical velocity w
  call write_2d(nulgmt, cpref, 'w_NH', csuff, cfmt2e,  &
    &  zw  (nx1:nx2, nz1 - 1:nz2), zx(nx1:nx2),  &
    &  zz_h(nx1:nx2, nz1 - 1:nz2), nx, nz + 1)

endif

! -----
! deallocate gridpoint fields
! -----

deallocate(zorog, zorogm, zx)
deallocate(zphi_f, zphi_h, zpi_f, zpi_h)
deallocate(zp, zq, zt, zu, zv, zvd_hy, zw_hy, ztheta)
deallocate(zz_f, zz_h)

if ( lnhdyn ) then
  deallocate(zpd, zvd, zw)
endif

! -----

! successful end
stop

end program flux

